/*  KRunning - EventEdit.cpp
 *  Copyright (C) 2002 by Andreas Scherf
 *  Begin Mon Jul 8 2002 write to scherfa@web.de if you found a bug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "EventEdit.h"

using namespace std;

EventListItem::EventListItem( QListViewItem *parent, eventObj *event )
        : QListViewItem( parent )
{
    myEvent = event;

    if (myEvent->getStartnumberFrames().length()>0) {
        int framenum=1;
        QStringList framelist = QStringList::split( ';', myEvent->getStartnumberFrames() );
        for ( QStringList::Iterator it = framelist.begin(); it != framelist.end(); ++it ) {
            QListViewItem *frames = new QListViewItem (this);
            frames->setText(0,i18n("Frame")+QString(" %1").arg(framenum++));
            frames->setText(1,*it);
        }
    }
    setText( 0, myEvent->getEventName() );
    QListViewItem *dist= new QListViewItem (this);
    dist->setText(0,i18n("Distance"));
    dist->setText(1,QString("%1").arg(myEvent->getEventDistance()));
    QListViewItem *start = new QListViewItem (this);
    start->setText(0,i18n("Starttime"));
    start->setText(1,myEvent->eventstarttime("hh:mm:ss"));
}

EventListItem::~EventListItem() {
}

EventEdit::EventEdit (QWidget * parent, const char *name) : EventEditGUI(parent,name,true) {
    ListView1->setColumnWidthMode (1, QListView::Maximum );
    ListView1-> setRootIsDecorated ( TRUE);
    ListView1->addColumn    ( i18n( "Details" ));
    ListView1->setItemsRenameable(true);
    ListView1->setRenameable (0,true);
    ListView1->setRenameable (1,true);
    UnitCombo->clear();
    UnitCombo->insertItem(i18n("km"),-1);
    UnitCombo->insertItem(i18n("m"),-1);
    UnitCombo->insertItem(i18n("miles"),-1);
    BlockGroupBox->setEnabled(false);
    connect ((QObject*)DistanceEdit,SIGNAL(returnPressed()),this, SLOT(distanceReturnPressed()) );
    //    connect ((QObject*)StartNumberCheck,SIGNAL(toggled(bool)),this, SLOT(startNumberCheckToggled(bool)) );
    connect ((QObject*)NewEventButton, SIGNAL( clicked() ),this, SLOT( newEventClicked() ) );
    connect ((QObject*)AddBlockButton,SIGNAL(clicked()),this,SLOT(addBlockClicked()));
    connect (ListView1, SIGNAL( selectionChanged( QListViewItem* ) ),  this, SLOT( eventSelected( QListViewItem* )));

    connect(ListView1,SIGNAL(itemRenamed (QListViewItem *,const QString &,int)),this,SLOT(slotEventChanged(QListViewItem *,const QString &,int)));

    QToolTip::add(DistanceEdit,i18n("Enter the distance of the course\nand press enter to save the new event."));
    eventObject=0;
}

EventEdit::~EventEdit() {
    if (eventObject)
        delete (eventObject);
}

void EventEdit::slotEventChanged(QListViewItem *item,const QString &name,int col) {
    QString frameblock;
    QStringList events;
    // DBConnection::eventObj -> save (true) means update ...
    //eventObj event=dbconn->getEventByName(item->text(0));

    if (item->text(0).contains(i18n("Distance"),true)>0) {
        eventObject->setDistance(name.toInt());
    } else if (item->text(0).contains(i18n("Starttime"),true)>0) {
        eventObject->setStartTime(QTime::fromString(name));
    }

    events=dbconn->getEventNameList();
    if (! events.grep(item->text(0)).isEmpty()) {
        eventObject->setEventName(name);
    }

    eventObject->save(true);
    cerr <<  item->text(0) << "-" << item->text(1) <<"-" << name <<  endl;

}

QListViewItem * EventEdit::findListItem(const QString &name) {
    QListViewItem *data = new QListViewItem (mainItem);
    return data;
}

void EventEdit::setDefaultSettings() {
    BlockGroupBox->setEnabled(false);
    setEnableEditFields(false);
    initListView();
}

void EventEdit::initListView () {
    ListView1->clear();
    DBConnection::EventList list;
    mainItem = new QListViewItem( ListView1, 0 );
    mainItem->setText( 0, dbconn->getDatabaseName() );
    mainItem->setSelectable(false);
    mainItem->setOpen(true);
    list=dbconn->getEventList();
    for (  DBConnection::EventList::Iterator it = list.begin(); it != list.end(); ++it ) {
        eventObj *event=new eventObj(*it);
        EventListItem *name = new EventListItem (mainItem,event);
        name->setPixmap( 0, QPixmap( KGlobal::dirs()->findResourceDir("pictures", "runner24.png")+"runner24.png"));
    }
}

void EventEdit::eventSelected( QListViewItem *i )
{
    EventListItem *item;
    if ( !i || i->depth()<1) {
        BlockGroupBox->setEnabled(false);
        setEnableEditFields(false);
        return;
    }

    if (i->depth()==1) {
        item = ( EventListItem* )i;
    } else {
        item = ( EventListItem* )i->parent();
        if (i->text(0).contains(i18n("Frame"),true)>0) {
            BlockGroupBox->setEnabled(true);
            FromNumInput->setValue( i->text(1).section('-',0,0).toInt());
            ToNumInput->setValue( i->text(1).section('-',-1).toInt());
        } else {
            BlockGroupBox->setEnabled(false);
        }
    }

    EventNameEdit->setText(item->event()->getEventName());
    TimeEdit->setTime(QTime::fromString(item->event()->eventstarttime("hh:mm:ss")));
    DistanceEdit->setText(QString("%1").arg(item->event()->getEventDistance()));
    eventObject = new eventObj(*item->event());
    setEnableEditFields(true);   
}

void EventEdit::distanceReturnPressed() {
    eventObj *event=new eventObj(0,EventNameEdit->text(),"",TimeEdit->time() ,DistanceEdit->text().toInt());
    event->save();       // Tree update und nummer muss eindeutig sein
    newItem->replaceEvent(event);
    ListView1->clear();
    initListView();
    setEnableEditFields(false);
    // in der View muss nun die Event Liste angepasst werden
    emit newEventCreated();
}


void EventEdit::newEventClicked() {
    eventObj *event=new eventObj(-1,i18n("new Event") ,"",QTime::currentTime(),0);
    newItem= new EventListItem (mainItem,event);
    newItem->setSelected(TRUE);
    eventSelected((QListViewItem *)newItem);
    ListView1->sort();
    ListView1->triggerUpdate();
    setEnableEditFields(true);
    EventNameEdit->setFocus();
    EventNameEdit->selectAll();
}

void EventEdit::addBlockClicked() {
    if (ListView1->selectedItem()==0) return;
    QString frame = QString("%1-%2").arg(FromNumInput->value()).arg(ToNumInput->value());
    if (dbconn->testStartNumberFrame(frame) != -1) return;  // Check if this frame exists or a number from this frame ...
    eventObj event=dbconn->getEventByName( ListView1->selectedItem()->text(0));
    event.addStartNumberFrame(frame);
    event.save(true); // Update event
    initListView();  // Only Item update !!
}

void EventEdit::setEnableEditFields(bool state) {
    EventNameEdit->setEnabled(state);
    TimeEdit->setEnabled(state);
    DistanceEdit->setEnabled(state);
    UnitCombo->setEnabled(state);
}
